@@ -1,5 +1,6 @@
 #!/usr/bin/env python
 
+import atexit
 import os
 import sys
 import re
@@ -70,6 +71,24 @@ CY3_DIR = None
 from distutils.command.build_ext import build_ext as _build_ext
 from distutils import sysconfig
 
+_to_clean = []
+
+@atexit.register
+def _cleanup_files():
+    """
+    This is only used on Cygwin to clean up shared libraries that are unsafe
+    to delete while the test suite is running.
+    """
+
+    for filename in _to_clean:
+        if os.path.isdir(filename):
+            shutil.rmtree(filename, ignore_errors=True)
+        else:
+            try:
+                os.remove(filename)
+            except OSError:
+                pass
+
 
 def get_distutils_distro(_cache=[]):
     if _cache:
@@ -431,39 +450,44 @@ class build_ext(_build_ext):
             pass
         _build_ext.build_extension(self, ext)
 
+
 class ErrorWriter(object):
     match_error = re.compile('(warning:)?(?:.*:)?\s*([-0-9]+)\s*:\s*([-0-9]+)\s*:\s*(.*)').match
+
     def __init__(self):
         self.output = []
         self.write = self.output.append
 
-    def _collect(self, collect_errors, collect_warnings):
+    def _collect(self):
         s = ''.join(self.output)
-        result = []
-        for line in s.split('\n'):
+        results = {'errors': [], 'warnings': []}
+        for line in s.splitlines():
             match = self.match_error(line)
             if match:
                 is_warning, line, column, message = match.groups()
-                if (is_warning and collect_warnings) or \
-                        (not is_warning and collect_errors):
-                    result.append( (int(line), int(column), message.strip()) )
-        result.sort()
-        return [ "%d:%d: %s" % values for values in result ]
+                results['warnings' if is_warning else 'errors'].append((int(line), int(column), message.strip()))
+
+        return [["%d:%d: %s" % values for values in sorted(results[key])] for key in ('errors', 'warnings')]
 
     def geterrors(self):
-        return self._collect(True, False)
+        return self._collect()[0]
 
     def getwarnings(self):
-        return self._collect(False, True)
+        return self._collect()[1]
 
     def getall(self):
-        return self._collect(True, True)
+        return self._collect()
+
+    def close(self):
+        pass  # ignore, only to match file-like interface
+
 
 class TestBuilder(object):
     def __init__(self, rootdir, workdir, selectors, exclude_selectors, annotate,
                  cleanup_workdir, cleanup_sharedlibs, cleanup_failures,
                  with_pyregr, cython_only, languages, test_bugs, fork, language_level,
-                 common_utility_dir):
+                 test_determinism,
+                 common_utility_dir, pythran_dir=None):
         self.rootdir = rootdir
         self.workdir = workdir
         self.selectors = selectors
@@ -478,7 +502,9 @@ class TestBuilder(object):
         self.test_bugs = test_bugs
         self.fork = fork
         self.language_level = language_level
+        self.test_determinism = test_determinism
         self.common_utility_dir = common_utility_dir
+        self.pythran_dir = pythran_dir
 
     def build_suite(self):
         suite = unittest.TestSuite()
@@ -562,10 +588,8 @@ class TestBuilder(object):
         return suite
 
     def build_tests(self, test_class, path, workdir, module, expect_errors, tags):
-        if 'werror' in tags['tag']:
-            warning_errors = True
-        else:
-            warning_errors = False
+        warning_errors = 'werror' in tags['tag']
+        expect_warnings = 'warnings' in tags['tag']
 
         if expect_errors:
             if 'cpp' in tags['tag'] and 'cpp' in self.languages:
@@ -583,14 +607,15 @@ class TestBuilder(object):
             languages.remove('cpp')
 
         preparse_list = tags.get('preparse', ['id'])
-        tests = [ self.build_test(test_class, path, workdir, module, tags,
-                                  language, expect_errors, warning_errors, preparse)
+        tests = [ self.build_test(test_class, path, workdir, module, tags, language,
+                                  expect_errors, expect_warnings, warning_errors, preparse,
+                                  self.pythran_dir if language == "cpp" else None)
                   for language in languages
                   for preparse in preparse_list ]
         return tests
 
-    def build_test(self, test_class, path, workdir, module, tags,
-                   language, expect_errors, warning_errors, preparse):
+    def build_test(self, test_class, path, workdir, module, tags, language,
+                   expect_errors, expect_warnings, warning_errors, preparse, pythran_dir):
         language_workdir = os.path.join(workdir, language)
         if not os.path.exists(language_workdir):
             os.makedirs(language_workdir)
@@ -601,6 +626,7 @@ class TestBuilder(object):
                           language=language,
                           preparse=preparse,
                           expect_errors=expect_errors,
+                          expect_warnings=expect_warnings,
                           annotate=self.annotate,
                           cleanup_workdir=self.cleanup_workdir,
                           cleanup_sharedlibs=self.cleanup_sharedlibs,
@@ -609,14 +635,18 @@ class TestBuilder(object):
                           fork=self.fork,
                           language_level=self.language_level,
                           warning_errors=warning_errors,
-                          common_utility_dir=self.common_utility_dir)
+                          test_determinism=self.test_determinism,
+                          common_utility_dir=self.common_utility_dir,
+                          pythran_dir=pythran_dir)
+
 
 class CythonCompileTestCase(unittest.TestCase):
     def __init__(self, test_directory, workdir, module, tags, language='c', preparse='id',
-                 expect_errors=False, annotate=False, cleanup_workdir=True,
+                 expect_errors=False, expect_warnings=False, annotate=False, cleanup_workdir=True,
                  cleanup_sharedlibs=True, cleanup_failures=True, cython_only=False,
                  fork=True, language_level=2, warning_errors=False,
-                 common_utility_dir=None):
+                 test_determinism=False,
+                 common_utility_dir=None, pythran_dir=None):
         self.test_directory = test_directory
         self.tags = tags
         self.workdir = workdir
@@ -625,6 +655,7 @@ class CythonCompileTestCase(unittest.TestCase):
         self.preparse = preparse
         self.name = module if self.preparse == "id" else "%s_%s" % (module, preparse)
         self.expect_errors = expect_errors
+        self.expect_warnings = expect_warnings
         self.annotate = annotate
         self.cleanup_workdir = cleanup_workdir
         self.cleanup_sharedlibs = cleanup_sharedlibs
@@ -633,11 +664,13 @@ class CythonCompileTestCase(unittest.TestCase):
         self.fork = fork
         self.language_level = language_level
         self.warning_errors = warning_errors
+        self.test_determinism = test_determinism
         self.common_utility_dir = common_utility_dir
+        self.pythran_dir = pythran_dir
         unittest.TestCase.__init__(self)
 
     def shortDescription(self):
-        return "compiling (%s) %s" % (self.language, self.name)
+        return "compiling (%s%s) %s" % (self.language, "/pythran" if self.pythran_dir is not None else "", self.name)
 
     def setUp(self):
         from Cython.Compiler import Options
@@ -673,8 +706,10 @@ class CythonCompileTestCase(unittest.TestCase):
         cleanup = self.cleanup_failures or self.success
         cleanup_c_files = WITH_CYTHON and self.cleanup_workdir and cleanup
         cleanup_lib_files = self.cleanup_sharedlibs and cleanup
+        is_cygwin = sys.platform == 'cygwin'
+
         if os.path.exists(self.workdir):
-            if cleanup_c_files and cleanup_lib_files:
+            if cleanup_c_files and cleanup_lib_files and not is_cygwin:
                 shutil.rmtree(self.workdir, ignore_errors=True)
             else:
                 for rmfile in os.listdir(self.workdir):
@@ -683,17 +718,32 @@ class CythonCompileTestCase(unittest.TestCase):
                                 rmfile[-4:] == ".cpp" or
                                 rmfile.endswith(".html") and rmfile.startswith(self.module)):
                             continue
-                    if not cleanup_lib_files and (rmfile.endswith(".so") or rmfile.endswith(".dll")):
+
+                    is_shared_obj = rmfile.endswith(".so") or rmfile.endswith(".dll")
+
+                    if not cleanup_lib_files and is_shared_obj:
                         continue
+
                     try:
                         rmfile = os.path.join(self.workdir, rmfile)
                         if os.path.isdir(rmfile):
                             shutil.rmtree(rmfile, ignore_errors=True)
+                        elif is_cygwin and is_shared_obj:
+                            # Delete later
+                            _to_clean.append(rmfile)
                         else:
                             os.remove(rmfile)
                     except IOError:
                         pass
 
+                if cleanup_c_files and cleanup_lib_files and is_cygwin:
+                    # Finally, remove the work dir itself
+                    _to_clean.append(self.workdir)
+
+        if cleanup_c_files and os.path.exists(self.workdir + '-again'):
+            shutil.rmtree(self.workdir + '-again', ignore_errors=True)
+
+
     def runTest(self):
         self.success = False
         self.runCompileTest()
@@ -702,7 +752,7 @@ class CythonCompileTestCase(unittest.TestCase):
     def runCompileTest(self):
         return self.compile(
             self.test_directory, self.module, self.workdir,
-            self.test_directory, self.expect_errors, self.annotate)
+            self.test_directory, self.expect_errors, self.expect_warnings, self.annotate)
 
     def find_module_source_file(self, source_file):
         if not os.path.exists(source_file):
@@ -744,25 +794,24 @@ class CythonCompileTestCase(unittest.TestCase):
     def split_source_and_output(self, test_directory, module, workdir):
         source_file = self.find_module_source_file(os.path.join(test_directory, module) + '.pyx')
         source_and_output = io_open(source_file, 'rU', encoding='ISO-8859-1')
+        error_writer = warnings_writer = None
         try:
             out = io_open(os.path.join(workdir, module + os.path.splitext(source_file)[1]),
-                              'w', encoding='ISO-8859-1')
+                          'w', encoding='ISO-8859-1')
             for line in source_and_output:
                 if line.startswith("_ERRORS"):
                     out.close()
-                    out = ErrorWriter()
+                    out = error_writer = ErrorWriter()
+                elif line.startswith("_WARNINGS"):
+                    out.close()
+                    out = warnings_writer = ErrorWriter()
                 else:
                     out.write(line)
         finally:
             source_and_output.close()
 
-        try:
-            geterrors = out.geterrors
-        except AttributeError:
-            out.close()
-            return []
-        else:
-            return geterrors()
+        return (error_writer.geterrors() if error_writer else [],
+                warnings_writer.geterrors() if warnings_writer else [])
 
     def run_cython(self, test_directory, module, targetdir, incdir, annotate,
                    extra_compile_options=None):
@@ -801,6 +850,7 @@ class CythonCompileTestCase(unittest.TestCase):
             annotate = annotate,
             use_listing_file = False,
             cplus = self.language == 'cpp',
+            np_pythran = self.pythran_dir is not None,
             language_level = self.language_level,
             generate_pxi = False,
             evaluate_tree_assertions = True,
@@ -824,15 +874,15 @@ class CythonCompileTestCase(unittest.TestCase):
                 build_extension.compiler = COMPILER
 
             ext_compile_flags = CFLAGS[:]
-            compiler = COMPILER or sysconfig.get_config_var('CC')
 
-            if self.language == 'c' and compiler == 'gcc':
-                ext_compile_flags.extend(['-std=c89', '-pedantic'])
             if  build_extension.compiler == 'mingw32':
                 ext_compile_flags.append('-Wno-format')
             if extra_extension_args is None:
                 extra_extension_args = {}
 
+            if self.pythran_dir is not None:
+                ext_compile_flags.extend(['-I',self.pythran_dir,'-DENABLE_PYTHON_MODULE','-std=c++11','-D__PYTHRAN__=%d' % sys.version_info.major,'-Wno-cpp'])
+
             related_files = self.related_files(test_directory, module)
             self.copy_files(test_directory, workdir, related_files)
 
@@ -894,10 +944,10 @@ class CythonCompileTestCase(unittest.TestCase):
         return get_ext_fullpath(module)
 
     def compile(self, test_directory, module, workdir, incdir,
-                expect_errors, annotate):
-        expected_errors = errors = ()
-        if expect_errors:
-            expected_errors = self.split_source_and_output(
+                expect_errors, expect_warnings, annotate):
+        expected_errors = expected_warnings = errors = warnings = ()
+        if expect_errors or expect_warnings:
+            expected_errors, expected_warnings = self.split_source_and_output(
                 test_directory, module, workdir)
             test_directory = workdir
 
@@ -906,9 +956,24 @@ class CythonCompileTestCase(unittest.TestCase):
             try:
                 sys.stderr = ErrorWriter()
                 self.run_cython(test_directory, module, workdir, incdir, annotate)
-                errors = sys.stderr.geterrors()
+                errors, warnings = sys.stderr.getall()
             finally:
                 sys.stderr = old_stderr
+            if self.test_determinism and not expect_errors:
+                workdir2 = workdir + '-again'
+                os.mkdir(workdir2)
+                self.run_cython(test_directory, module, workdir2, incdir, annotate)
+                diffs = []
+                for file in os.listdir(workdir2):
+                    if (open(os.path.join(workdir, file)).read()
+                        != open(os.path.join(workdir2, file)).read()):
+                        diffs.append(file)
+                        os.system('diff -u %s/%s %s/%s > %s/%s.diff' % (
+                            workdir, file,
+                            workdir2, file,
+                            workdir2, file))
+                if diffs:
+                    self.fail('Nondeterministic file generation: %s' % ', '.join(diffs))
 
         tostderr = sys.__stderr__.write
         if 'cerror' in self.tags['tag']:
@@ -919,23 +984,10 @@ class CythonCompileTestCase(unittest.TestCase):
                 tostderr('\n\n')
                 raise RuntimeError('should have generated extension code')
         elif errors or expected_errors:
-            try:
-                for expected, error in zip(expected_errors, errors):
-                    self.assertEquals(expected, error)
-                if len(errors) < len(expected_errors):
-                    expected_error = expected_errors[len(errors)]
-                    self.assertEquals(expected_error, None)
-                elif len(errors) > len(expected_errors):
-                    unexpected_error = errors[len(expected_errors)]
-                    self.assertEquals(None, unexpected_error)
-            except AssertionError:
-                tostderr("\n=== Expected errors: ===\n")
-                tostderr('\n'.join(expected_errors))
-                tostderr("\n\n=== Got errors: ===\n")
-                tostderr('\n'.join(errors))
-                tostderr('\n\n')
-                raise
+            self._match_output(expected_errors, errors, tostderr)
             return None
+        if expected_warnings or (expect_warnings and warnings):
+            self._match_output(expected_warnings, warnings, tostderr)
 
         so_path = None
         if not self.cython_only:
@@ -971,6 +1023,24 @@ class CythonCompileTestCase(unittest.TestCase):
                         tostderr("\n==============================\n")
         return so_path
 
+    def _match_output(self, expected_output, actual_output, write):
+        try:
+            for expected, actual in zip(expected_output, actual_output):
+                self.assertEquals(expected, actual)
+            if len(actual_output) < len(expected_output):
+                expected = expected_output[len(actual_output)]
+                self.assertEquals(expected, None)
+            elif len(actual_output) > len(expected_output):
+                unexpected = actual_output[len(expected_output)]
+                self.assertEquals(None, unexpected)
+        except AssertionError:
+            write("\n=== Expected: ===\n")
+            write('\n'.join(expected_output))
+            write("\n\n=== Got: ===\n")
+            write('\n'.join(actual_output))
+            write('\n\n')
+            raise
+
 
 class CythonRunTestCase(CythonCompileTestCase):
     def setUp(self):
@@ -982,7 +1052,7 @@ class CythonRunTestCase(CythonCompileTestCase):
         if self.cython_only:
             return CythonCompileTestCase.shortDescription(self)
         else:
-            return "compiling (%s) and running %s" % (self.language, self.name)
+            return "compiling (%s%s) and running %s" % (self.language, "/pythran" if self.pythran_dir is not None else "", self.name)
 
     def run(self, result=None):
         if result is None:
@@ -1450,7 +1520,6 @@ class EmbedTest(unittest.TestCase):
         os.chdir(self.old_dir)
 
     def test_embed(self):
-        from distutils import sysconfig
         libname = sysconfig.get_config_var('LIBRARY')
         libdir = sysconfig.get_config_var('LIBDIR')
         if not os.path.isdir(libdir) or libname not in os.listdir(libdir):
@@ -1776,6 +1845,10 @@ def main():
                       help="deterministic generated by string")
     parser.add_option("--use_common_utility_dir", default=False, action="store_true")
     parser.add_option("--use_formal_grammar", default=False, action="store_true")
+    parser.add_option("--test_determinism", default=False, action="store_true",
+                      help="test whether Cython's output is deterministic")
+    parser.add_option("--pythran-dir", dest="pythran_dir", default=None,
+                      help="specify Pythran include directory. This will run the C++ tests using Pythran backend for Numpy")
 
     options, cmd_args = parser.parse_args(args)
 
@@ -1971,10 +2044,17 @@ def runtests(options, cmd_args, coverage=None):
         exclude_selectors.append(ShardExcludeSelector(options.shard_num, options.shard_count))
 
     if not test_bugs:
+        bug_files = [
+            ('bugs.txt', True),
+            ('pypy_bugs.txt', IS_PYPY),
+            ('windows_bugs.txt', sys.platform == 'win32'),
+            ('cygwin_bugs.txt', sys.platform == 'cygwin')
+        ]
+
         exclude_selectors += [
-            FileListExcluder(os.path.join(ROOTDIR, bugs_file_name), verbose=verbose_excludes)
-            for bugs_file_name in ['bugs.txt'] + (['pypy_bugs.txt'] if IS_PYPY else []) +
-            (['windows_bugs.txt'] if sys.platform == 'win32' else [])
+            FileListExcluder(os.path.join(ROOTDIR, bugs_file_name),
+                             verbose=verbose_excludes)
+            for bugs_file_name, condition in bug_files if condition
         ]
 
     if sys.platform in ['win32', 'cygwin'] and sys.version_info < (2,6):
@@ -2024,7 +2104,8 @@ def runtests(options, cmd_args, coverage=None):
                                 options.pyregr,
                                 options.cython_only, languages, test_bugs,
                                 options.fork, options.language_level,
-                                common_utility_dir)
+                                options.test_determinism,
+                                common_utility_dir, options.pythran_dir)
         test_suite.addTest(filetests.build_suite())
 
     if options.system_pyregr and languages:
@@ -2038,7 +2119,8 @@ def runtests(options, cmd_args, coverage=None):
                                     True,
                                     options.cython_only, languages, test_bugs,
                                     options.fork, sys.version_info[0],
-                                    common_utility_dir)
+                                    options.test_determinism,
+                                    common_utility_dir, options.pythran_dir)
             sys.stderr.write("Including CPython regression tests in %s\n" % sys_pyregr_dir)
             test_suite.addTest(filetests.handle_directory(sys_pyregr_dir, 'pyregr'))
 
